#ifndef CONSTANTS
#define CONSTANTS

// Bowell 1989 HG simple constants
#define HG_SIMPLE_A1 3.33
#define HG_SIMPLE_A2 1.87
#define HG_SIMPLE_B1 0.63
#define HG_SIMPLE_B2 1.22

// Bowell 1989 HG constants
#define HG_A1 3.332
#define HG_A2 1.862
#define HG_B1 0.631
#define HG_B2 1.218
#define HG_C1 0.986
#define HG_C2 0.238

// Muinonen 2010 HG1G2/HG12 constants
#define M_1_PI 0.3183098861837906715377675267450287
#define M_1_5PI 0.0636619772367581343075535053490057

// Muinonen 2010 HG12 constants
#define G1_LT_SCAL 0.7527
#define G1_LT_CONST 0.06164
#define G1_GTE_SCAL 0.9529
#define G1_GTE_CONST 0.02162

#define G2_LT_SCAL -0.9612
#define G2_LT_CONST 0.6270
#define G2_GTE_SCAL -0.6125
#define G2_GTE_CONST 0.5572


#endif